/*
** Copyright (C) 3 January 2000 George Saj <gsaj@acm.org>
**
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/
#include <gtk/gtk.h>
#include <signal.h>
#include <stdlib.h>
#include <stdio.h>
#include <errno.h> 
#include "psinfo.h"
#include "gui.h"


/* macros */
#define PREFS_GUI(voidptr) ((PrefsGui *) voidptr)

/* static and global variables */
static gchar *user_list_buttons[3] = {"Show All Users",
				      "Show Only Your Processes", 
				      "Show Those in Filter List:" };
static gint selected_signal;
static GuiElements main_gui;
static SignalStruct signal_list[NUM_SIGNALS] = 
{{"SIGHUP - 1",SIGHUP},
 {"SIGINT - 2",SIGINT},
 {"SIGQUIT - 3",SIGQUIT},
 {"SIGABRT - 6",SIGABRT},
 {"SIGKILL - 9",SIGKILL},
 {"SIGALRM - 14",SIGALRM},
 {"SIGTERM - 15",SIGTERM},
 {"SIGCHLD - 17",SIGCHLD},
 {"SIGCONT - 18",SIGCONT},
 {"SIGSTOP - 19",SIGSTOP}
};
/**/

/* declaration for DEFAULT columns set */
gboolean default_cols[NUMCOL] = {1, 1, 0, 1, 0, 0, 1, 0, 1, 0, 1};
appCfg appstate; /* application configuration structure */
PrefsGui *prefs; /* it is easier to have this here. THIS MEANS A BIT
			  OF RESTRUCTURING TO CLEAN THIS UP A BIT! */

/* function prototypes */
/* Callbacks */
void user_select_add(GtkWidget* button, gpointer data);
void user_select_del(GtkWidget* button, gpointer data);
void user_select_clear(GtkWidget* button, gpointer data);
void proc_scan(GtkWidget *w, gpointer data);
void pref_radio_toggle_all(GtkWidget *w, gpointer data);
void pref_radio_toggle_cur(GtkWidget *w, gpointer data);
void pref_radio_toggle_pck(GtkWidget *w, gpointer data);
void close_prefs_dialog(GtkWidget *widget, gpointer data);
void close_simple_dialog(GtkWidget *widget, gpointer data);
void remove_grab(GtkWidget *widget, gpointer data);
void display_about_dialog(GtkWidget *w, gpointer data);
void keep_prefs(GtkWidget *widget, gpointer data);
void column_check_toggle(GtkWidget *checkitem, gpointer data);
void select_row_callback(GtkWidget *widget, gint row, gint column, 
			 GdkEventButton *event, gpointer data);
void select_signal_callback(GtkWidget *widget, gpointer data);
void send_signal(GtkWidget *widget, gpointer data);
gint DestroyWindow(GtkWidget *widget, gpointer *data);
void select_clist_col(GtkWidget *w, gint col, gpointer data);

/* other functions */
GtkWidget *create_simple_dialog(gchar *title);
void clear_gstring_list(GList *inList);
GList *copy_ulist(GList *src, GList *dest);
void init_prefs_userclist();
GtkWidget *get_main_menu(GtkWidget *window);
void create_sigopmenu();
void create_clist();
void error_dialog1(gchar *text);
void set_clist_sens(gboolean sens);
gint clist_col_cmp(GtkCList *inList, gconstpointer ptr1, 
		   gconstpointer ptr2);

GtkWidget* ulist_toggle_button_init(GtkWidget *w, gshort labelnum);
GtkWidget *init_clist();
void prefs_dialog_init();
void gui_init();
/**/


/* For the menu */
static GtkItemFactoryEntry menu_items[] = {
  {"/_File",         NULL,         NULL, 0, "<Branch>"},
  {"/File/_Rescan",    "<control>R", proc_scan, 0, NULL},
  {"/File/sep1",     NULL,         NULL, 0, "<Separator>"},
  {"/File/Quit",     "<control>Q", gtk_main_quit, 0, NULL},
  {"/_Options",      NULL,         NULL, 0, "<Branch>"},
  {"/Options/Preferences...", NULL, prefs_dialog_init, 0, NULL},
  {"/Options/sep2",     NULL,         NULL, 0, "<Separator>"},
  {"/Options/Save options",     "<control>S", 0, 0, NULL},
  {"/_Help",         NULL,         NULL, 0, "<LastBranch>"},
  {"/_Help/About",   NULL,         display_about_dialog, 0, NULL},
};
/***************/

void close_prefs_dialog(GtkWidget *widget, gpointer data){

     gtk_widget_destroy (GTK_WIDGET (PREFS_GUI(data)->dialog));
     g_free(data); // frees the prefs struct
}

void close_simple_dialog(GtkWidget *widget, gpointer data){
     gtk_widget_destroy (GTK_WIDGET(data));

}
void remove_grab(GtkWidget *widget, gpointer data){
     gtk_grab_remove(GTK_WIDGET(widget));
}

/* keep in mind this dialog creator doesn't show the dialog on its own */
GtkWidget *create_simple_dialog(gchar *title) {
     GtkWidget *dialog;
     dialog = gtk_dialog_new();
     gtk_signal_connect(GTK_OBJECT (dialog), "destroy",
			GTK_SIGNAL_FUNC(remove_grab), &dialog);
     gtk_window_set_title (GTK_WINDOW(dialog), title);
     return dialog;
     
}

void display_about_dialog(GtkWidget *w, gpointer data){
    
     GtkWidget *about_dialog, *label1, *label2, *button;

     about_dialog = create_simple_dialog("About");
//add by nico
#ifdef PDA
     gtk_widget_set_usize(about_dialog, 240, 300);
#endif
     gtk_container_border_width (GTK_CONTAINER (about_dialog), 5);

     label1 = gtk_label_new(GTASKMAN_VERSION);
     label2 = gtk_label_new(AUTHOR_TEXT);

     gtk_box_pack_start(GTK_BOX(GTK_DIALOG (about_dialog)->vbox),
			label1, TRUE, TRUE, 0);
     gtk_box_pack_start(GTK_BOX(GTK_DIALOG (about_dialog)->vbox),
			label2, TRUE, TRUE, 0);
     gtk_widget_show(label1);
     gtk_widget_show(label2);
     
     button = gtk_button_new_with_label("OK");
     gtk_signal_connect (GTK_OBJECT(button),"clicked",
			 GTK_SIGNAL_FUNC(close_simple_dialog),
			 about_dialog);

     GTK_WIDGET_SET_FLAGS(button, GTK_CAN_DEFAULT);
     
     gtk_box_pack_start (GTK_BOX (GTK_DIALOG (about_dialog)->action_area),
			 button, TRUE,TRUE,0);
     gtk_widget_grab_default(button);
     gtk_widget_show(button);
     gtk_widget_show(about_dialog);
     gtk_grab_add(about_dialog);

}

void keep_prefs(GtkWidget *widget, gpointer data){
    int i;
    GList * delTemp;
    /* save column list */
    for (i = 0; i < NUMCOL; i++) {
	appstate.showed_columns[i] = prefs->col[i].column_val;
    } 
    /* save user_filter option */
    appstate.ulist_option = prefs->ulist_opt;

    /* save user list for PCK_USR (from UserListEnum) */
    delTemp = appstate.inc_ulist;
    if (delTemp)
	clear_gstring_list(delTemp);
    appstate.inc_ulist = prefs->ulist;
    /* close the dialog */
    close_prefs_dialog(widget, data);

    /* reload */
    proc_rescan(appstate.inc_ulist);
}

void set_clist_sens(gboolean sens){

    gtk_widget_set_sensitive(GTK_WIDGET(prefs->userClist), sens);
    gtk_widget_set_sensitive(GTK_WIDGET(prefs->uname_entry), sens);
    gtk_widget_set_sensitive(GTK_WIDGET(prefs->add_button), sens);
    gtk_widget_set_sensitive(GTK_WIDGET(prefs->del_button), sens);
    gtk_widget_set_sensitive(GTK_WIDGET(prefs->clr_button), sens);

}
void column_check_toggle(GtkWidget *checkitem, gpointer data){
/* callback to handle checkitems for column filtering getting clicked */
     if (GTK_TOGGLE_BUTTON (checkitem)->active) 
	  ((Columnitem *)data)->column_val = 1;
     else
	  ((Columnitem *)data)->column_val = 0;
}

void pref_radio_toggle_all(GtkWidget *w, gpointer data){
    set_clist_sens(0);
    prefs->ulist_opt = ALL_USR;
}


void pref_radio_toggle_cur(GtkWidget *w, gpointer data){
    set_clist_sens(0);
    prefs->ulist_opt = CUR_USR; 
}


void pref_radio_toggle_pck(GtkWidget *w, gpointer data){
    set_clist_sens(1);
    prefs->ulist_opt = PCK_USR;     
}

void user_select_add(GtkWidget *widget, gpointer data){

 /*input validation is still needed here!! */
 /* also, needs handling duplicate adds - if already in list, don't add it! */
    GString *newuser = g_string_sized_new(5); /* default size of 5 */
    g_string_assign(newuser, gtk_entry_get_text(GTK_ENTRY(
	PREFS_GUI(data)->uname_entry)));
    PREFS_GUI(data)->ulist = g_list_append(PREFS_GUI(data)->ulist, newuser);
    gtk_clist_append(GTK_CLIST(PREFS_GUI(data)->userClist), 
		     &(newuser->str)); 
    gtk_entry_set_text(GTK_ENTRY(prefs->uname_entry), "");/* clear entry box */
}

void user_select_del(GtkWidget* widget, gpointer data){
    GList *iter;
    GString* rmStr = g_string_sized_new(8);
    gint retFromTxt; /* thrown away, just holds a return val below*/
    if (PREFS_GUI(data)->selected_list_item != -1 && prefs->ulist){
	retFromTxt = gtk_clist_get_text(GTK_CLIST(PREFS_GUI(data)->userClist),
					   PREFS_GUI(data)->selected_list_item,
					   0, &(rmStr->str));
	for (iter = prefs->ulist; iter != NULL; iter = iter->next) {
	    if (!strcmp(rmStr->str, ((GString *)iter->data)->str)){
		prefs->ulist = g_list_remove(prefs->ulist, iter->data);
		break;
	    }
	}
	gtk_clist_remove(GTK_CLIST(PREFS_GUI(data)->userClist), 
			 PREFS_GUI(data)->selected_list_item);
	
	PREFS_GUI(data)->selected_list_item = -1;
    }

    g_string_free(rmStr, TRUE); // free the allocated string
    if (!g_list_length(prefs->ulist))
	prefs->ulist = NULL;
	
    gtk_entry_set_text(GTK_ENTRY(prefs->uname_entry), "");/* clear entry box */
}


void user_select_clear(GtkWidget* widget, gpointer data){
     /* flush the list */
    gtk_clist_clear(GTK_CLIST(PREFS_GUI(data)->userClist));
    PREFS_GUI(data)->selected_list_item = -1;

    /* clear the prefs->ulist */
    clear_gstring_list(PREFS_GUI(data)->ulist);
    prefs->ulist = NULL;
    prefs->selected_list_item = -1;
}

void clear_gstring_list(GList *inList){
    GList *iter;
    for (iter = inList; iter != NULL; iter = iter->next){
	  if (iter->data != NULL)
	  g_string_free((GString *)iter->data, TRUE);
    }
    g_list_free(inList);
    inList = NULL;
}

gint clist_col_cmp(GtkCList *inCList, gconstpointer ptr1, 
			  gconstpointer ptr2) {
    gdouble precision;
    gint i;
    GtkCListRow *row1 = (GtkCListRow *) ptr1;
    GtkCListRow *row2 = (GtkCListRow *) ptr2;
    gchar *Str1 = NULL, *Str2 = NULL, *testStr = NULL;


    Str1 = GTK_CELL_TEXT(row1->cell[appstate.clist_sort_col])->text;
    Str2 = GTK_CELL_TEXT(row2->cell[appstate.clist_sort_col])->text;
    
   
    switch (appstate.clist_sort_meth){
    case Sort_Meth_Num:
	    precision = g_strtod(Str1,NULL) - g_strtod(Str2,NULL);
	    if (precision < 0)
		return -1;
	    else if (precision > 0)
		return 1;
	    else
		return 0;
    case (Sort_Meth_Str || Sort_Meth_Time): /* time needs its own   */
	return(strcmp(Str1,Str2));          /* to be accurate. Fix! */
    }
}
void select_clist_col(GtkWidget *inCList, gint col, gpointer data){
/* here we do column sorting for the clicked row */
    
    gchar *testStr = NULL;
    gtk_clist_set_sort_column(GTK_CLIST(inCList), col);
    if (appstate.clist_sort_col == col) {
	if (appstate.clist_sort_type == GTK_SORT_ASCENDING)
	    appstate.clist_sort_type = GTK_SORT_DESCENDING;
	else
	    appstate.clist_sort_type = GTK_SORT_ASCENDING;
	gtk_clist_set_sort_type(GTK_CLIST(inCList), appstate.clist_sort_type);
    }
    else { /* default to ascending sort */
	    appstate.clist_sort_col = col;	
	    appstate.clist_sort_type = GTK_SORT_ASCENDING;
	    gtk_clist_set_sort_type(GTK_CLIST(inCList), GTK_SORT_ASCENDING);
    }

    /* get an item from the column to test for string/numbers */   
    gtk_clist_get_text(GTK_CLIST(inCList), 0, appstate.clist_sort_col, 
		       &testStr);
    
    if (testStr[2] == ':')
	appstate.clist_sort_meth = Sort_Meth_Time; 
    else if (isdigit(testStr[0]))
	appstate.clist_sort_meth = Sort_Meth_Num;
    else
	appstate.clist_sort_meth = Sort_Meth_Str;

    gtk_clist_set_compare_func(GTK_CLIST(inCList), 
			       clist_col_cmp);

    gtk_clist_freeze(GTK_CLIST(inCList));
    gtk_clist_sort(GTK_CLIST(inCList));
    gtk_clist_thaw(GTK_CLIST(inCList));
}

void init_prefs_userclist(){
    GList *iter;
    
    prefs->userClist = gtk_clist_new (1); /* yep, a 1 column clist! */
    gtk_clist_column_titles_hide(GTK_CLIST(prefs->userClist));
    gtk_container_add(GTK_CONTAINER(prefs->scrollwindow),prefs->userClist); 
    gtk_signal_connect(GTK_OBJECT(prefs->userClist),"select_row",
		       select_row_callback, &(prefs->selected_list_item));

    for (iter = prefs->ulist; iter != NULL; iter = iter->next){
	gtk_clist_append(GTK_CLIST(prefs->userClist), 
			 &(((GString *)iter->data)->str));
    }
     
     gtk_widget_show (prefs->userClist);
}


GtkWidget* ulist_toggle_button_init(GtkWidget *w, gshort labelnum){
    w = gtk_radio_button_new_with_label (prefs->uid_rgroup,
					 (user_list_buttons[labelnum]));
    prefs->uid_rgroup = gtk_radio_button_group (GTK_RADIO_BUTTON (w));
/*    gtk_object_set_data (GTK_OBJECT (prefs->dialog), 
			 user_list_buttons[labelnum], 
			 w);*/

    if (labelnum == prefs->ulist_opt)
	 gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(w), TRUE); 

    gtk_widget_show (w);
    gtk_box_pack_start (GTK_BOX(prefs->vbox2),
			w, FALSE, FALSE, 0);

    return w;   
}

GList *copy_ulist(GList *src, GList *dest){
    GList *iter;
    dest = NULL; /* initialize the destination ptr */
    for (iter = src; iter !=NULL; iter = iter->next){
	GString *cur_str = g_string_new(((GString*)iter->data)->str);
	dest = g_list_append(dest, cur_str);
    }

    return dest;
}

void prefs_dialog_init(){
     GtkWidget *label, *fixed1;
     gint i;     
     prefs = g_malloc (sizeof(PrefsGui));
     prefs->notebook = gtk_notebook_new();
     prefs->vbox1 = gtk_vbox_new(FALSE, 0);
     
     prefs->ulist = copy_ulist(appstate.inc_ulist, prefs->ulist);

     prefs->ulist_opt = appstate.ulist_option;
     
     prefs->selected_list_item = -1; /* nothing selected yet */

     for (i = 0; i < NUMCOL; i++) {

	  prefs->col[i].checkitem = gtk_check_button_new_with_label(columntitles[i]);
	  prefs->col[i].column_num = i;
	  prefs->col[i].column_val = appstate.showed_columns[i];
	  gtk_box_pack_start(GTK_BOX(prefs->vbox1),prefs->col[i].checkitem, 
			     FALSE,FALSE,0);
	  gtk_signal_connect(GTK_OBJECT (prefs->col[i].checkitem), 
			     "toggled",
			     GTK_SIGNAL_FUNC(column_check_toggle),
			     &prefs->col[i]);
	  if (1 == prefs->col[i].column_val)
	       gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (prefs->col[i].checkitem),1);
	  gtk_widget_show(prefs->col[i].checkitem);
     }
     gtk_widget_show(prefs->vbox1);
     label = gtk_label_new("Columns");
     gtk_notebook_append_page(GTK_NOTEBOOK (prefs->notebook), 
			      prefs->vbox1, label );

     prefs->dialog = create_simple_dialog("Preferences");
//add by nico
#ifdef PDA
     gtk_widget_set_usize(prefs->dialog, 240, 300);
#endif     
     gtk_container_border_width(GTK_CONTAINER (prefs->dialog), 8);

     prefs->button = gtk_button_new_with_label("OK");

     GTK_WIDGET_SET_FLAGS(prefs->button, GTK_CAN_DEFAULT);

     gtk_box_pack_start(GTK_BOX (GTK_DIALOG (prefs->dialog)->action_area), 
			prefs->button, TRUE, TRUE, 0);
     gtk_signal_connect(GTK_OBJECT (prefs->button), "clicked",
			GTK_SIGNAL_FUNC (keep_prefs), 
			prefs);
     gtk_widget_show(prefs->button);
     gtk_widget_grab_default(prefs->button);
     
     prefs->button = gtk_button_new_with_label("Cancel");
     gtk_box_pack_start(GTK_BOX (GTK_DIALOG (prefs->dialog)->action_area), prefs->button, TRUE, TRUE, 0);
     gtk_signal_connect(GTK_OBJECT (prefs->button), "clicked",
			GTK_SIGNAL_FUNC (close_prefs_dialog),
			prefs);
     gtk_widget_show(prefs->button);
     
     gtk_box_pack_start(GTK_BOX(GTK_DIALOG (prefs->dialog)->vbox),
			prefs->notebook, TRUE, TRUE, 0);
     /* okay, that's ONE notebook page */
     
     prefs->vbox2 = gtk_vbox_new(FALSE,0);
     gtk_widget_show (prefs->vbox2);

     label = gtk_label_new("User List");
     gtk_notebook_append_page(GTK_NOTEBOOK (prefs->notebook), 
			      prefs->vbox2, label );/*init next notebook page*/
     gtk_widget_show(label);
     prefs->uid_rgroup = NULL; /* initialize radio group */

     /* radio buttons, Page 2 */
     /* calling a function to config them */
     prefs->radiobutton1 = ulist_toggle_button_init(prefs->radiobutton1, 0);
     prefs->radiobutton2 = ulist_toggle_button_init(prefs->radiobutton2, 1);
     prefs->radiobutton3 = ulist_toggle_button_init(prefs->radiobutton3, 2);
     
     /* set toggle callbacks for radio buttons */
     gtk_signal_connect(GTK_OBJECT (prefs->radiobutton1),"toggled",
			GTK_SIGNAL_FUNC(pref_radio_toggle_all), prefs);
     gtk_signal_connect(GTK_OBJECT (prefs->radiobutton2),"toggled", 
			GTK_SIGNAL_FUNC(pref_radio_toggle_cur), prefs);
     gtk_signal_connect(GTK_OBJECT (prefs->radiobutton3),"toggled", 
			GTK_SIGNAL_FUNC(pref_radio_toggle_pck), prefs);

    
     
     prefs->hbox1 = gtk_hbox_new(FALSE, 0);
     gtk_box_pack_start(GTK_BOX(prefs->vbox2), 
			prefs->hbox1, FALSE, FALSE, 0);
     gtk_widget_show(prefs->hbox1);

     /* Add button */
     prefs->add_button = gtk_button_new_with_label("Add");
     gtk_signal_connect(GTK_OBJECT (prefs->add_button), "clicked",
			GTK_SIGNAL_FUNC(user_select_add),
			prefs);
     gtk_box_pack_start(GTK_BOX(prefs->hbox1), 
			prefs->add_button, FALSE, FALSE, 0);
     gtk_widget_show(prefs->add_button);
      
     /* entry box */
     prefs->uname_entry = gtk_entry_new_with_max_length(8);
//add by nico
#ifdef PDA
     gtk_widget_set_usize(prefs->uname_entry,120,25);
#endif     
     gtk_box_pack_start (GTK_BOX (prefs->hbox1), prefs->uname_entry, FALSE, FALSE, 0);
     gtk_entry_set_editable(GTK_ENTRY(prefs->uname_entry), TRUE);
     gtk_widget_show (prefs->uname_entry);

     /* Del button */
     prefs->del_button = gtk_button_new_with_label("Del");
     gtk_box_pack_start(GTK_BOX(prefs->hbox1), 
			prefs->del_button, FALSE, FALSE, 0);
     gtk_signal_connect(GTK_OBJECT (prefs->del_button), "clicked",
			GTK_SIGNAL_FUNC(user_select_del),
			prefs);
     gtk_widget_show(prefs->del_button);
     /* Clear button */
     prefs->clr_button = gtk_button_new_with_label("Clear");
     gtk_box_pack_start(GTK_BOX(prefs->hbox1), 
			prefs->clr_button, FALSE, FALSE, 0);
      gtk_signal_connect(GTK_OBJECT (prefs->clr_button), "clicked",
			 GTK_SIGNAL_FUNC(user_select_clear),
			 prefs);
     gtk_widget_show(prefs->clr_button);
     
    
     prefs->scrollwindow = gtk_scrolled_window_new (NULL, NULL);
//add by nico
#ifdef PDA
    gtk_widget_set_usize(prefs->scrollwindow,200,100);
#endif
     gtk_widget_show (prefs->scrollwindow);
     gtk_box_pack_start(GTK_BOX(prefs->vbox2), 
			prefs->scrollwindow, FALSE, FALSE, 0);
     gtk_container_set_border_width (GTK_CONTAINER (prefs->scrollwindow),
				     2);
     gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW 
				     (prefs->scrollwindow), 
				     GTK_POLICY_AUTOMATIC, 
				     GTK_POLICY_AUTOMATIC);
     init_prefs_userclist();

     if (appstate.ulist_option == ALL_USR || appstate.ulist_option == CUR_USR)
	 set_clist_sens(0);
     
     gtk_widget_show(prefs->notebook);
     gtk_widget_show(prefs->dialog);
     gtk_grab_add(prefs->dialog);
     
}


void proc_scan(GtkWidget *w, gpointer data){
  gtk_widget_set_sensitive(main_gui.sigopmenu, FALSE );
  proc_rescan(appstate.inc_ulist);
}


GtkWidget *get_main_menu(GtkWidget *window) {
    int nmenu_items = sizeof(menu_items) / sizeof(menu_items[0]);
    GtkWidget *menubar;
    GtkItemFactory *item_factory;
    GtkAccelGroup *accel_group;
    accel_group = gtk_accel_group_new();
    item_factory = gtk_item_factory_new(GTK_TYPE_MENU_BAR, "<main>",
					accel_group);
    
    gtk_item_factory_create_items(item_factory, nmenu_items, menu_items, NULL);
    gtk_accel_group_attach (accel_group, GTK_OBJECT (window));
    
    menubar = gtk_item_factory_get_widget(item_factory, "<main>");
    gtk_widget_show(menubar);
    return menubar;
}

void select_row_callback(GtkWidget *widget, gint row,
			 gint column, GdkEventButton *event,
			 gpointer data) {
/* data is a ptr to the integer you want to set the row to. */
    if (widget == main_gui.clist1) /*check pointer for main_gui*/
	gtk_widget_set_sensitive(main_gui.sigopmenu, TRUE );
    *(gshort *)data = (gshort) row;
}

void select_signal_callback(GtkWidget *widget, gpointer data) {
     
     selected_signal = *(gint *)data;
}

void create_sigopmenu(){

     gshort i;

     main_gui.sigopmenu = gtk_option_menu_new ();
     gtk_object_set_data (GTK_OBJECT (main_gui.window), "sigopmenu", main_gui.sigopmenu);
     gtk_widget_show (main_gui.sigopmenu);
     main_gui.sigmenu = gtk_menu_new ();

     for (i = 0; i < NUM_SIGNALS; i++){
	  main_gui.sigmenuitem = 
	       gtk_menu_item_new_with_label (signal_list[i].sig_name);
	  gtk_menu_append (GTK_MENU (main_gui.sigmenu),
			   main_gui.sigmenuitem);
	  gtk_signal_connect(GTK_OBJECT(main_gui.sigmenuitem),
                        "activate",
                        select_signal_callback,
                        &signal_list[i].sig_val);
	  gtk_widget_show (main_gui.sigmenuitem);
     }

     gtk_option_menu_set_menu (GTK_OPTION_MENU (main_gui.sigopmenu), 
			       main_gui.sigmenu);
     gtk_option_menu_set_history (GTK_OPTION_MENU (main_gui.sigopmenu), 6);
     gtk_widget_set_sensitive(main_gui.sigopmenu, FALSE );
}


void create_clist(){
     int i;
     main_gui.clist1 = gtk_clist_new(calc_columns());
     
     /* for column filtering */
     gtk_clist_column_titles_active(GTK_CLIST(main_gui.clist1));
     /* set the signals for clist buttons */

     for (i = 0; i < calc_columns(); i++)
	  gtk_clist_set_column_auto_resize(GTK_CLIST (main_gui.clist1),i,1);
}

GtkWidget* init_clist(){
/* clear the clist */

     if (main_gui.clist1 != NULL) {
	  gtk_widget_hide(main_gui.clist1);
	  gtk_container_remove(GTK_CONTAINER(main_gui.clist1->parent),
			       main_gui.clist1); /* I believe this Destroys 
						    the clist, too */
     } 
     
     create_clist();

     gtk_container_add(GTK_CONTAINER(main_gui.scrollwindow), 
	  main_gui.clist1);
     
     gtk_signal_connect(GTK_OBJECT(main_gui.clist1),"select_row",
                        select_row_callback, &main_gui.selected_row);
     gtk_signal_connect(GTK_OBJECT(main_gui.clist1),"click-column",
			select_clist_col, prefs);
     return main_gui.clist1;
}


void error_dialog1(gchar *text){
     GtkWidget *errdialog, *button, *errmsg;

     errdialog = GTK_WIDGET (create_simple_dialog("Error"));
     gtk_container_border_width (GTK_CONTAINER (errdialog), 10);
     errmsg = gtk_label_new(text);

     gtk_box_pack_start(GTK_BOX(GTK_DIALOG (errdialog)->vbox),
			errmsg, TRUE, TRUE, 0);
     gtk_widget_show(errmsg);
     
     button = gtk_button_new_with_label("OK");
     gtk_signal_connect (GTK_OBJECT(button),"clicked",
			 GTK_SIGNAL_FUNC(close_simple_dialog),
			 errdialog);
     GTK_WIDGET_SET_FLAGS(button, GTK_CAN_DEFAULT);
     
     gtk_box_pack_start (GTK_BOX (GTK_DIALOG (errdialog)->action_area),
			 button, TRUE,TRUE,0);
     gtk_widget_grab_default(button);
     gtk_widget_show(button);
     gtk_widget_show(errdialog);
     gtk_grab_add(errdialog);
     proc_rescan(appstate.inc_ulist);
}


void send_signal(GtkWidget *widget, gpointer data){
     gchar *pid_str;
     int kill_result = 0;

     gtk_clist_get_text(GTK_CLIST(main_gui.clist1),main_gui.selected_row,
			1, &pid_str);
   
     kill_result = kill((pid_t)strtod(pid_str,0), selected_signal);
     if (kill_result == -1) {
	  gchar failure[100]; 
	  snprintf(failure, sizeof(failure), "error: %s", strerror(errno));
	  error_dialog1(failure);
     }    
     else
	  proc_rescan(appstate.inc_ulist);
}
void set_showed_columns(gboolean* inCols) {
    gshort i;
    for (i = 0; i < NUMCOL; i++)
	appstate.showed_columns[i] = inCols[i];
    
}


void gui_init() {

    /* init appstate - this will change to reading .gtaskmanrc to get
       results or using defaults. For now it's just defaults */
    
    appstate.inc_ulist = NULL;
    appstate.ulist_option = CUR_USR;
    set_showed_columns(default_cols);
    appstate.clist_sort_col = appstate.showed_columns[1]; 
    appstate.clist_sort_type = GTK_SORT_ASCENDING;
/* end of appstate init */

    /* main gui config begins here */
    main_gui.window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
    gtk_window_set_title(GTK_WINDOW (main_gui.window), "gtaskman");
//add by nico
#ifdef PDA
    gtk_widget_set_usize(main_gui.window, 240, 300);
#else
    gtk_widget_set_usize(main_gui.window, 426, 300);
#endif
    gtk_container_border_width(GTK_CONTAINER (main_gui.window), 0);
    gtk_signal_connect (GTK_OBJECT (main_gui.window), "delete_event",
			GTK_SIGNAL_FUNC (DestroyWindow), NULL);
    
    main_gui.vbox1 = gtk_vbox_new(FALSE, 0);
    gtk_container_border_width(GTK_CONTAINER(main_gui.vbox1), 1);
    
    main_gui.menubar = get_main_menu(main_gui.window);
    gtk_box_pack_start(GTK_BOX(main_gui.vbox1), main_gui.menubar, FALSE, 
		       TRUE, 0);
    
    main_gui.scrollwindow = gtk_scrolled_window_new(NULL, NULL);
    
    gtk_scrolled_window_set_policy((GtkScrolledWindow *) main_gui.scrollwindow, 
				   GTK_POLICY_ALWAYS, GTK_POLICY_AUTOMATIC);
    
    gtk_widget_show(main_gui.scrollwindow);
    
    
    proc_rescan(); /* Makes initial call to create table of processes */
    
    gtk_box_pack_start(GTK_BOX (main_gui.vbox1), main_gui.scrollwindow, 
		       TRUE, TRUE, 0);
    
    main_gui.separator = gtk_hseparator_new ();
    gtk_box_pack_start(GTK_BOX(main_gui.vbox1), main_gui.separator, FALSE, 
		       FALSE, 0);
    gtk_widget_show (main_gui.separator);
    
    /* bottom buttons */
    main_gui.hbox1 = gtk_hbox_new(FALSE,0);
    gtk_container_border_width(GTK_CONTAINER(main_gui.hbox1), 3);
    
    main_gui.button = gtk_button_new_with_label("Signal");
    gtk_signal_connect(GTK_OBJECT (main_gui.button), "clicked",
		       GTK_SIGNAL_FUNC (send_signal),
		       GTK_OBJECT (main_gui.clist1));
    gtk_box_pack_start(GTK_BOX (main_gui.hbox1), main_gui.button, TRUE, TRUE, 0);
    gtk_widget_show(main_gui.button);
    
    create_sigopmenu();
    
    gtk_box_pack_start(GTK_BOX (main_gui.hbox1), main_gui.sigopmenu, TRUE, 
		       TRUE, 0);
    gtk_widget_show(main_gui.button);
    
    main_gui.button = gtk_button_new_with_label("Quit");
    gtk_signal_connect_object (GTK_OBJECT (main_gui.button), "clicked",
			       GTK_SIGNAL_FUNC (gtk_main_quit),
			       GTK_OBJECT (main_gui.window));
    
    gtk_box_pack_start(GTK_BOX (main_gui.hbox1), main_gui.button, TRUE, 
		       TRUE, 0);
    gtk_widget_show(main_gui.button);
    
  /* final packin' */
    gtk_box_pack_start(GTK_BOX (main_gui.vbox1), main_gui.hbox1, FALSE, 
		       FALSE, 0);
    gtk_widget_show(main_gui.hbox1);
    
    gtk_container_add(GTK_CONTAINER (main_gui.window), main_gui.vbox1);
    
    gtk_widget_show (main_gui.vbox1);
    gtk_widget_show  (main_gui.window);
}

/* Window destruction callback */
gint DestroyWindow(GtkWidget *widget, gpointer *data){
  gtk_main_quit();
  return FALSE;
}












