/*
** Copyright (C) 3 January 2000 George Saj <gsaj@acm.org>
**
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/


 /* clist sorting advice in GTK-app-devel list 9/29 */

/* make a GtkMenu that does the same as the optionmenu 
   centralize the sigopmenu from gtaskman.c - get it out of there! */

#include <stdio.h>
#include <stdlib.h>
#include <string.h> 
#include <unistd.h>
#include <gtk/gtk.h>

#ifdef OS_LINUX
#include <sys/types.h> /* linux specific? */
#endif

#include <pwd.h> /* needed to match username to uid */
#include "psinfo.h"
#include "gui.h"

gchar columntitles[NUMCOL][8];

#ifdef OS_LINUX
#define TIME_COL 9
#endif

/****** function prototypes *******/
gchar** g_strsplit_gs (const gchar *string, const gchar *delimiter,
                       const gint  max_tokens);
gint calc_columns();
void populate_clist(char *ps_lines);
void rescan_linux();
/*********************************/

gchar** g_strsplit_gs (const gchar *string,
                       const gchar *delimiter,
                       const gint  max_tokens)
/* This is a modified version of the glib function by a similar name
   The glib version doesn't handle two or more delimiters
   in a row. This removes "leading delimiters" on each token
   It's meant for single character delimiters. . .
   and also has a hack in it to handle where linux 2.0 has dates like 
   "Jul 20" on processes that have been running for some time.
   I'm not sure if that hack works yet. . .
 */
{
  GSList *string_list = NULL, *slist;
  gchar **str_array, *s;
  guint i, n = 1;
  gint num_tokens = max_tokens;
  g_return_val_if_fail (string != NULL, NULL);
  g_return_val_if_fail (delimiter != NULL, NULL);
  if (max_tokens < 1)
       num_tokens = G_MAXINT;
  s = strstr (string, delimiter);
  while(s && delimiter[0] == s[1])
       s++;
  if (s)
    {
      guint delimiter_len = strlen (delimiter);
      do
        {
	  guint len;
          gchar *new_string;
          if (n == max_tokens) /* keeps last token intact */
               len = strlen (string);
          else
               len = s - string;
          new_string = g_new (gchar, len + 1);
          strncpy (new_string, string, len);
          new_string[len] = 0;
          string_list = g_slist_prepend (string_list, new_string);
          n++;
          string = s + delimiter_len;
          s = strstr (string, delimiter);
#ifdef OLDDATEFIX /* for linux 2.0 issues with day-old+ procs. . . */
	    if (n == TIME_COL) { 
	       if ((strncasecmp(string, "jan ",4) == 0 ||
		    strncasecmp(string, "feb ",4) == 0 ||
		    strncasecmp(string, "mar ",4) == 0 ||
		    strncasecmp(string, "apr ",4) == 0 ||
		    strncasecmp(string, "may ",4) == 0 ||
		    strncasecmp(string, "jun ",4) == 0 ||
		    strncasecmp(string, "jul ",4) == 0 ||
		    strncasecmp(string, "aug ",4) == 0 ||
		    strncasecmp(string, "sep ",4) == 0 ||
		    strncasecmp(string, "oct ",4) == 0 ||
		    strncasecmp(string, "nov ",4) == 0 ||
		    strncasecmp(string, "dec ",4) == 0)){
		    string = string + 3;
	       }     
	    }
#endif
          while(s && delimiter[0] == s[1])
               s++;
        } while (--num_tokens && s);
    }
  if (*string)
    {
      n++;
      string_list = g_slist_prepend (string_list, g_strdup (string));
    }
  str_array = g_new (gchar*, n);
  i = n -1;
  str_array[i--] = NULL;
  for (slist = string_list; slist; slist = slist->next)
    str_array[i--] = slist->data;
  g_slist_free (string_list);
  return str_array;
  }



gint calc_columns(){
     gint i, num_columns = 0;
     for (i = 0; i < NUMCOL; i++)
	  if (appstate.showed_columns[i] == 1)
	       num_columns++;
     return num_columns;
}

void populate_clist(char *ps_lines){
	/* uList is a pointer to the prefs list to check against */

    struct passwd* userinfo = getpwuid(getuid()); /* this gets the username from uid */
    char *cUsername = userinfo->pw_name;
    gchar **all_lines, **tokens;
    gchar *inserted_tokens[calc_columns()];
    gchar *word;
    gint row = 0, col = 0, last_col = 0;
    GtkWidget *clist1 = init_clist();
    
    /* first split up the lines */
    all_lines = g_strsplit(ps_lines, NEWLINE, 0); /* 0 for no max */
    
    /* first line contains column headings */
    tokens = g_strsplit_gs(all_lines[row++], SPACE, NUMCOL);
    for (col = 0; col < NUMCOL; col++) {
	g_strdown(g_strchomp(tokens[col]));
	strncpy(columntitles[col],tokens[col],strlen(tokens[col]));
	if (appstate.showed_columns[col] == 1) {
	    gtk_clist_set_column_title (GTK_CLIST(clist1), 
					last_col++, tokens[col]);
	}
    }
    g_strfreev(tokens);
    gtk_clist_column_titles_show(GTK_CLIST(clist1));
     /* now handle the rest of the tokens */
    while (all_lines[row] != NULL) {
	last_col = 0;
	tokens = g_strsplit_gs(all_lines[row], SPACE, NUMCOL);
	g_strchomp(tokens[0]);
	
	for (col = 0; col < NUMCOL; col++) {
	    if (appstate.showed_columns[col] == 1) 
		inserted_tokens[last_col++] = tokens[col];
	}
	
	switch(appstate.ulist_option){
	    GList *iter;
	case ALL_USR: /* this is default behavior */
	    if (!((gint) g_strtod(tokens[1],0) == getpid())
		&& (strcmp(PSCMD, tokens[10])))
		gtk_clist_append(GTK_CLIST(clist1),inserted_tokens);
	    
	    break;
	case CUR_USR: 
	    if (!strcmp(tokens[0],cUsername))
      /* tests for gtaskman's PID and doesn't let it into the clist */
		/* if (appstate.display_gtask_proc) */ /* user pref */
		if (!((gint) g_strtod(tokens[1],0) == getpid())
		    && (strcmp(PSCMD, tokens[10])))
		    gtk_clist_append(GTK_CLIST(clist1),inserted_tokens);
	    break;
	case PCK_USR:
	    
	    for (iter = appstate.inc_ulist; iter != NULL; iter = iter->next){
		if (!strcmp(tokens[0], ((GString *)iter->data)->str)){
		    if (!((gint) g_strtod(tokens[1],0) == getpid())
			&& (strcmp(PSCMD, tokens[10])))
			gtk_clist_append(GTK_CLIST(clist1),inserted_tokens);
		    break;
		}
	    }
	    break;
	}
		
	g_strfreev(tokens); 	  
	row++;     
    }
    
    /* after done with the arrays, free hem using g_strfreev() */   
    g_strfreev(all_lines);
    gtk_widget_show(clist1);
}


#ifdef OS_LINUX
void proc_rescan(GList *uList) {
    
    FILE *read_ps;
    char buffer[BUFSIZ + 1];
    int chars_read;
    
    char *line;
    
    memset(buffer, '\0',sizeof(buffer));
    read_ps = popen(PSCMD, "r"); 
    
    if (read_ps != NULL) {
	chars_read = fread(buffer, sizeof(char), BUFSIZ, read_ps);
	pclose(read_ps);
	
	populate_clist(buffer);
    }
    else
	g_error("ps produced no output!\nIf your ps works, report this to the maintainer <gsaj@acm.org>");
}   
#endif


#ifdef OS_SUNOS 
/* sunos support to come */
#endif

#ifdef OS_FBSD
/* freebsd support to come */
#endif













